#!/usr/bin/perl

## Chris Dagdigian <chris@bioteam.net> 
##
##  For background on why this script may be useful
##  please visit http://wiki.gridengine.info/wiki/index.php/GridEngine_launchd
##
##  Summary:
##   This script creates system start/stop scripts for Grid Engine 
##   that are compatible with the new OS X 10.5 "Leopard" launchd
##   framework. Apple is strongly pushing launchd and has threatened
##   to deprecate SystemStarter and /etc/rc.common in the future. 
## 
##   This script is compatible for SGE versions 6.0 through 6.1 
##   it should work for SGE 6.2 as long as the sgeschedd.plist files
##   are deleted (sge_schedd is no longer an independent daemon in 6.2)

unless($ENV{'SGE_ROOT'}) {  
  die "Error: Please initialize the Grid Engine environment, $SGE_ROOT not found.";
 }

$ARCH = `/bin/sh $ENV{'SGE_ROOT'}/util/arch`;
chomp($ARCH);
unless( $ARCH == "darwin-x86" || 
	$ARCH == "darwin-ppc" ||
	$ARCH == "darwin-unsupported" ) { 
    die "Error: Unable to detect an Apple OS X architecture";    
}

my $dataStart = tell DATA;
my @daemons   = qw(qmaster schedd execd);

foreach $item (@daemons) {
    print "Creating launchd plist file for: sge_$item\n";
    open(FH,">./net.sunsource.gridengine.sge$item.plist") || die "Can't write to current directory, $!";

    while (<DATA>) {
	$line = $_;
	$line =~ s/xxPROGRAMxx/$item/g;
	$line =~ s/xxARCHxx/$ARCH/g;
	$line =~ s/xxROOTxx/$ENV{'SGE_ROOT'}/g;
	$line =~ s/xxCELLxx/$ENV{'SGE_CELL'}/g;
	print FH $line;
    }
    close(FH);
    seek DATA, $dataStart, 0;
}

print <<EOF

File completion complete.

Usage:
 The .plist files are ready to be copied into /Library/LaunchDaemons/ on
 each apple system in your cluster.

 Please make sure the sgeqmaster.plist and sgeschedd.plist files are only
 installed on your qmaster host and the sgeexecd.plist files only on named
 SGE execution hosts.

Start on boot:

 Anything in /Library/LaunchDaemons/ will be started at system boot time
 
Manual Control

 - First the .plist files need to be loaded:

 # launchctl load /Library/LaunchDaemons/net.sunsource.gridengine.sgeqmaster.plist
 # launchctl load /Library/LaunchDaemons/net.sunsource.gridengine.sgeschedd.plist
 # launchctl load /Library/LaunchDaemons/net.sunsource.gridengine.sgeexecd.plist

 - Once loaded SGE can be started via:

 # launchctl start /Library/LaunchDaemons/net.sunsource.gridengine.sgeqmaster.plist
 # launchctl start /Library/LaunchDaemons/net.sunsource.gridengine.sgeschedd.plist
 # launchctl start /Library/LaunchDaemons/net.sunsource.gridengine.sgeexecd.plist

 - And stopped via ...

 # launchctl unload /Library/LaunchDaemons/net.sunsource.gridengine.sgeqmaster.plist
 # launchctl unload /Library/LaunchDaemons/net.sunsource.gridengine.sgeschedd.plist
 # launchctl unload /Library/LaunchDaemons/net.sunsource.gridengine.sgeexecd.plist

 All of these commands can be used in shell scripts if you would like to create
 custom "sgeqmaster" and "sgeexecd" scripts that leverage the launchd framework.

EOF


__DATA__
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
	<key>Label</key>
	<string>net.sunsource.gridengine.sgexxPROGRAMxx</string>
	<key>Program</key>
	<string>xxROOTxx/bin/xxARCHxx/sge_xxPROGRAMxx</string>
	<key>RunAtLoad</key>
	<true/>
	<key>EnvironmentVariables</key>
	<dict>
		<key>SGE_ROOT</key>
		<string>xxROOTxx</string>
		<key>SGE_CELL</key>
		<string>xxCELLxx</string>
		<key>SGE_ND</key>
		<string>1</string>
		<key>DYLD_LIBRARY_PATH</key>
		<string>xxROOTxx/lib/xxARCHxx</string>
	</dict>
	<key>StandardErrorPath</key>
	<string>/dev/null</string>
	<key>StandardOutPath</key>
	<string>/dev/null</string>
    <key>KeepAlive</key>
    <true/>
</dict>
</plist>
